//      Yasuharu Okamoto, 2017.07

data {
    int <lower = 3> K;                          //      Number of categories
    int Npsn;                                   //      Number of persons
    int Nitm;                                   //      Number of items
    int Ntot;                                   //      Number of data,  Ntot = Nprn * Nitm
    int<lower = 1, upper = Npsn> IDpsn[Ntot];   //      Person ID
    int<lower = 1, upper = Nitm> IDitm[Ntot];   //      Item ID
    int<lower = 1, upper = K> Res[Ntot];        //      Response-> 1: No,   2: Yes
}

parameters {
        real<lower = 0.0> a[Nitm];
        real b[Nitm - 1];
        ordered[K - 1] c;
        real F[Npsn];
}

transformed parameters {
    vector[K] p[Ntot];
    vector[K] shft[Nitm];

    for (k in 1:(K-1)){
        shft[1][k] = c[k];
        for (j in 2:(Nitm)){
            shft[j][k] = shft[1][k] + b[j - 1];
        }
    }

    for (i in 1:Ntot){
        p[i][K] = inv_logit(1.7 * a[IDitm[i]] * (F[IDpsn[i]] - shft[IDitm[i]][K - 1]));
        p[i][1] = 1.0 - inv_logit(1.7 * a[IDitm[i]] * (F[IDpsn[i]] - shft[IDitm[i]][1]));
        for (k in 2:(K - 1)){
            p[i][k] = inv_logit(1.7 * a[IDitm[i]] * (F[IDpsn[i]] - shft[IDitm[i]][k-1]))
                        -inv_logit(1.7 * a[IDitm[i]] * (F[IDpsn[i]] - shft[IDitm[i]][k]));
        }
    }
}

model {
    for (i in 1:Npsn)
        F[i] ~ normal(0.0, 1.0);
    for (i in 1:Ntot)
        Res[i] ~ categorical(p[i]);
}
