#
#       Yasuharu Okamoto, 2017.07
#
import pystan
from pystan import StanModel
import pickle

def calc_mean_med( d ):
    d.sort()
    med = d[len(d) // 2]
    sum = 0
    for v in d:
        sum += v
    mean = sum / len(d)
    return mean, med

#
#       Prepare the input data file
#

fn_in = input("Input data file = ")

f_in = open(fn_in, "r")

#
#       Set the contents of the input data file in the object data_f
#

data_f = f_in.readlines()
f_in.close()

#
#       Prepare the output file
#

fn_out = input("Output data file = ")
f_out = open(fn_out, "w")
f_out.write("Input data file = " + fn_in + "\n\n")

pos = 0
while True:
    if len(data_f[pos]) > 0:
        if data_f[pos][0] == "/":
            break
    pos += 1

pos += 1
N = int(data_f[pos])
pos += 1
M = int(data_f[pos])
pos += 1
K = int(data_f[pos])
print('N = ', N, '   M = ', M, '   K = ', K)

X = []
for i in range(N):
    X.append([])
    pos += 1
    temp_strs = data_f[pos].split()
    for j in range(M):
        X[i].append(int(temp_strs[j + 1]))

IDpsn = []
IDitm = []
Res = []
for i in range(N):
    for j in range(M):
        IDpsn.append(i + 1)
        IDitm.append(j + 1)
        Res.append(X[i][j])
        

f_out.write('\nData...\n')
for i in range(N):
    f_out.write('{0:>5}: '.format(i+1))
    for j in range(M):
        f_out.write(' {0}'.format(X[i][j]))
    f_out.write('\n')

Data = {'K': K, 'Npsn': N, 'Nitm': M, 'Ntot': N * M, 'IDpsn': IDpsn, 'IDitm': IDitm, 'Res': Res}

fit = pystan.stan(file = 'irt_pol.stan', data = Data, seed = 999,
                  pars = ['a', 'b', 'c'], n_jobs = 1)

print(fit)
                
a = fit['a']
b = fit['b']
c = fit['c']

a_smpls = []
b_smpls = []
c_smpls = []

for i in range(M):
    a_smpls.append([])
for i in range(M-1):
    b_smpls.append([])
for k in range(K-1):
    c_smpls.append([])
for v in a:
    for i in range(M):
        a_smpls[i].append(v[i])
for v in b:
    for i in range(M-1):
        b_smpls[i].append(v[i])
for v in c:
    for i in range(K-1):
        c_smpls[i].append(v[i])

f_out.write('\n')
for i in range(M):
    mean_a, med_a = calc_mean_med(a_smpls[i])
    f_out.write('a[{0}] = {1:<.5}(mean)   {2:<.5}(med)\n'.
                format(i+1, mean_a, med_a))
f_out.write('\n')
f_out.write('b[1] = 0.0   (indentification condition)\n')
for i in range(M-1):
    mean_b, med_b = calc_mean_med(b_smpls[i])
    f_out.write('b[{0}] = {1:<.5}(mean)   {2:<.5}(med)\n'.
                format(i+2, mean_b, med_b)) 
f_out.write('\n')
for i in range(K-1):
    mean_c, med_c = calc_mean_med(c_smpls[i])
    f_out.write('c[{0}] = {1:<.5}(mean)   {2:<.5}(med)\n'.
                format(i+1, mean_c, med_c)) 

f_out.close()
